#include "Status.h"
#include "Vram.h"
#include "Main.h"
#include "Sprite.h"
#include "Print.h"
#include "Chars.h"

static ptr<byte> PrintS(ptr<byte> pVram, ptr<byte> p)
{
    byte c;
    while ((c = *p) != 0) {
        pVram = PrintC(pVram, c);
        ++p;
    }
    return pVram;
}

void PrintStatus() 
{
    PrintS(Vram + 1 * VramStep, "SCORE");
    // PrintS(Vram + 7 * VramStep, "HI-SCORE");
    PrintS(Vram + 7 * VramStep, "ST");
    PrintByteNumber2(Vram + VramRowSize + 7 * VramStep, CurrentStage + 1);
    PrintS(Vram+ 10 * VramStep, "TIME");
    {
        ptr<byte> pVram;
        pVram = Vram + 15 * VramStep;
        if (RemainCount > 1) {
            byte i;
            i = RemainCount - 1;
            do {
                Put2C(pVram, Char_Remain);
                pVram += 2 * VramStep;
                --i;
            } while (i > 0);
        }
    }

    PrintScore();
    PrintTime();
}

void PrintScore()
{
    ptr<byte> pVram;
    pVram = PrintNumber5(Vram + VramRowSize, Score);
    PrintC(pVram, '0');
    // pVram = PrintNumber5(Vram + VramRowSize + 9 * VramStep, HiScore);
    // PrintC(pVram, '0');
}

void PrintTime()
{
    PrintByteNumber3(Vram + VramRowSize + 11 * VramStep, StageTime);
}

// static void EraseWindow() 
// {
//     constexpr byte width = 12;
//     constexpr byte height = 3;    
//     ptr<byte> pVram;

//     pVram = Vram + VramRowSize * 11 + 10 * VramStep;
//     repeat (height) {
//         repeat (width) {
//             pVram = PrintC(pVram, ' ');
//         }
//         ppVram += VramRowSize - width;
//     }
// }

void PrintGameOver()
{
    // EraseWindow();
    PrintS(Vram + VramRowSize * 7 + (WindowWidth - 9) / 2 * VramStep, "GAME OVER");
}

void PrintTimeUp() 
{
    // EraseWindow();
    PrintS(Vram + VramRowSize * 7 + (WindowWidth - 7) / 2 * VramStep, "TIME UP");
}

void Title()
{
    ClearScreen(); 
    HideAllSprites();
    HScroll = 0;
    PrintStatus();
    {
        static const byte[] TitleBytes = {
            0x2c, 0x2a, 0x1f, 0x1d, 
            0x2c, 0x2b, 0x1e, 0x2b, 
            0x2c, 0x29, 0x20, 0x2c, 
            0x22, 0x22, 0x1d, 0x21, 
            0x1d, 0x1d, 0x1d, 0x1d, 
            0x26, 0x25, 0x24, 0x23, 
            0x29, 0x29, 0x20, 0x20, 
            0x1e, 0x1d, 0x22, 0x1d, 
            0x27, 0x1d, 0x1d, 0x1d, 
            0x2c, 0x1e, 0x2b, 0x1e, 
            0x2c, 0x1d, 0x2a, 0x20, 
            0x21, 0x1e, 0x22, 0x1d, 
            0x27, 0x1d, 0x1d, 0x1d, 
            0x2c, 0x2b, 0x1e, 0x2b, 
            0x2c, 0x28, 0x1d, 0x2c, 
            0x22, 0x21, 0x1e, 0x21, 
            0x1d, 0x25, 0x1f, 0x27, 
            0x26, 0x29, 0x20, 0x2c, 
            0x22, 0x29, 0x20, 0x2c, 
            0x22, 0x21, 0x1e, 0x22, 
        };
        constexpr byte LogoLength = 5;
        ptr<byte> p;
        ptr<byte> pVram;
        pVram = Vram + VramRowSize * 4 + (WindowWidth - 4 * LogoLength) / 2 * VramStep;
        p = TitleBytes;
        repeat (LogoLength) {
            repeat (4) {
                repeat (4) {
                    pVram = Put(pVram, *p);
                    ++p;
                }
                pVram += VramRowSize - 4 * VramStep;
            }
            pVram += 4 * VramStep - VramRowSize * 4;
        }
        PrintS(Vram + VramRowSize * 8 + ((WindowWidth + 4 * LogoLength) / 2 - 4) * VramStep, "MINI");
    }
    PrintS(Vram + VramRowSize * 13 + 1 * VramStep, "PUSH START BUTTON");
    PrintS(Vram + VramRowSize * 15 + 8 * VramStep, "INUFUTO 2024");
}


void DrawFence()
{
    ptr<byte> pVram;
    pVram = Vram + VramRowSize * 16;
    repeat(WindowWidth) {
        pVram = Put(pVram, Char_Fence);
    }
}